// Demo 11

//         Ulam's Conjecture
//  A Java Screen Based Application
//
//   by Charles W. Neville, 1998
// Copyright Charles W. Neville, 1998

//  Based on many similar programs written in Basic, C, or Pascal,
//    for example Julien Hennefeld's PROGRAM ULAM.

// Stanislaw Ulam was one of the brilliant mathematicians composing the Polish 
// School of Mathematics.  He, along with his colleagues such as Serpinski, 
// layed much of the foundation for point set topology in the 1920's and 
// 1930's.  A refugee from Nazi terror, he fled to the United States at the 
// beginning of the Second World War, where he contributed to the war effort 
// by working on the Manhattan Project.  After the war, he remained in the 
// United States, where he continued to make significant contributions to 
// mathematics.
//
// Several of his conjectures remain unresolved to this day.  (That is, we 
// still do not know whether they are true or false.)  Here is one which is 
// especially simple to state, but which has defied some of the best minds in 
// mathematics.  Start with any integer greater than 0.  If it is even, divide 
// by 2.  If it is odd, multiply by 3 and add 1.  Continue in this fashion. 
// Ulam's conjecture states that eventually, you will obtain 1.
//
// Try running this program on several starting integers, all greater than 0.
// See if you end up with 1.  If you don't, you are instantly famous.

import java.io.*;  // have to import Java's IO (Input/Output) classes to
                   // read from the keyboard, or to do any kind of file 
                   // based IO.

class Ulam {

   public static void main( String args[] ) throws IOException {

     // this is a screen based application, so we don't need to construct
     // an instance of ourselves, PROVIDED that all methods are declared
     // as static, and all main program variables are defined inside method
     // main (or declared as static).

     int x;

     // Abracadabra! Arcane stuff needed to deal with screen-keyboard based
     // input.  Just accept it for the time being.  Basically, what this
     // does is open the keyboard as a special kind of file (stream) which
     // is particularly easy to read from.  Treat the "throws IOException"
     // clause in the declaration of main the same way, just accept it for
     // the time being as needed whenever you do keyboard based input in
     // a screen based application, or read from a file.
     BufferedReader stdin;
     stdin = new BufferedReader( new InputStreamReader( System.in ) );

     // issue user prompt and get starting value from user
     System.out.print("Enter a starting integer > 0: ");
     System.out.flush();  // force system to write prompt to screen.  Not
                          // flushing after a print lets the system write
                          // to the screen whenever it pleases, which may
                          // be only after program has ended!  So ALWAYS
                          // FLUSH AFTER USING PRINT.
     x = Integer.parseInt( stdin.readLine() );
     while (x != 1) {

        // compute next integer using the Ulam process
        if ( x % 2 == 0)  {  // case where x is even

           x = x/2;
        }
        else {  // case where x is odd

           x = 3*x + 1;
        }
        System.out.println(x);
     }
   }
}





